import re
import unittest

from tf_core.nltoolkit.lib.textual_data_in_out import load_adc_from_tsv


class LoadDocumentCorpusFromTSV(unittest.TestCase):
    def test_load_adc_from_tsv(self):
        input_data = [('doc1', 'This is some text', '1'), ('doc2', 'This is some other text', '0')]
        header = ['title', 'text', 'label']

        input_text = []
        input_text.append('\t'.join(header))
        for line in input_data:
            input_text.append('\t'.join(line))

        input_dict = {
            'input': '\n'.join(input_text),
            'title_column': 'title',
            'text_column': 'text',
            'label_column': 'label'
        }
        output = load_adc_from_tsv(input_dict)
        adc = output['adc']

        self.assertEqual(len(input_data), len(adc.documents))
        for i, _ in enumerate(input_data):
            self.assertEqual(input_data[i][0], adc.documents[i].name)
            self.assertEqual(input_data[i][1], adc.documents[i].text)
            self.assertEqual(input_data[i][2], adc.documents[i].get_first_label())

    def test_custom_property_names(self):
        input_data = [('doc1', 'This is some text', '1'), ('doc2', 'This is some other text', '0')]
        header = ['some special title', 'some special text', 'some special label']

        input_text = []
        input_text.append('\t'.join(header))
        for line in input_data:
            input_text.append('\t'.join(line))

        input_dict = {
            'input': '\n'.join(input_text),
            'title_column': 'some special title',
            'text_column': 'some special text',
            'label_column': 'some special label'
        }
        output = load_adc_from_tsv(input_dict)
        adc = output['adc']

        self.assertEqual(len(input_data), len(adc.documents))
        for i, _ in enumerate(input_data):
            self.assertEqual(input_data[i][0], adc.documents[i].name)
            self.assertEqual(input_data[i][1], adc.documents[i].text)
            self.assertEqual(input_data[i][2], adc.documents[i].get_first_label())

    def test_multiple_spaces_in_text(self):
        input_data = [('doc1', 'This is                      some text', '1'),
                      ('doc2', 'This is some other text    ', '0')]
        header = ['title', 'text', 'label']

        input_text = []
        input_text.append('\t'.join(header))
        for line in input_data:
            input_text.append('\t'.join(line))

        input_dict = {
            'input': '\n'.join(input_text),
        }
        output = load_adc_from_tsv(input_dict)
        adc = output['adc']

        self.assertEqual(len(input_data), len(adc.documents))
        for i, _ in enumerate(input_data):
            self.assertEqual(input_data[i][0], adc.documents[i].name)
            # text gets normalized
            self.assertEqual(re.sub(' +', ' ', input_data[i][1]), adc.documents[i].text)
            self.assertEqual(input_data[i][2], adc.documents[i].get_first_label())

    def test_no_title(self):
        input_data = [('This is some text', '1'), ('This is some other text', '0')]
        header = ['text', 'label']

        input_text = []
        input_text.append('\t'.join(header))
        for line in input_data:
            input_text.append('\t'.join(line))

        input_dict = {
            'input': '\n'.join(input_text),
        }
        output = load_adc_from_tsv(input_dict)
        adc = output['adc']

        self.assertEqual(len(input_data), len(adc.documents))
        for i, _ in enumerate(input_data):
            self.assertEqual('', adc.documents[i].name)
            self.assertEqual(input_data[i][0], adc.documents[i].text)
            self.assertEqual(input_data[i][1], adc.documents[i].get_first_label())

    def test_no_label(self):
        input_data = [('doc1', 'This is some text'), ('doc2', 'This is some other text')]
        header = ['title', 'text']

        input_text = []
        input_text.append('\t'.join(header))
        for line in input_data:
            input_text.append('\t'.join(line))

        input_dict = {
            'input': '\n'.join(input_text),
        }
        output = load_adc_from_tsv(input_dict)
        adc = output['adc']

        self.assertEqual(len(input_data), len(adc.documents))
        for i, _ in enumerate(input_data):
            self.assertEqual(input_data[i][0], adc.documents[i].name)
            self.assertEqual(input_data[i][1], adc.documents[i].text)
            self.assertEqual('', adc.documents[i].get_first_label())

    def test_no_text(self):
        input_dict = {
            'input': 'some strange format',
        }
        self.assertRaises(Exception, load_adc_from_tsv, input_dict)


if __name__ == '__main__':
    unittest.main()



from .lib.bag_of_words import *
from .lib.classification import *
#from lib.clustering import *
from .lib.document_corpus import *
#from lib.similarity_matrix import *
from .lib.part_of_speech_tagging import *
from .lib.stop_word_removal import *
from .lib.stemming import *
from .lib.textual_data_in_out import *
from .lib.tokenization import *
from .lib.chunking import *


import os.path
import re
# from docx import opendocx, getdocumenttext # TODO update with python-docx
# from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
# from pdfminer.converter import TextConverter
# from pdfminer.layout import LAParams
# from pdfminer.pdfpage import PDFPage
# from io import StringIO
from subprocess import PIPE, Popen

# TODO Rewrite this part, pdfminer doesn't work under python 3
# def convert_pdf_to_txt(path):
#     rsrcmgr = PDFResourceManager()
#     retstr = StringIO()
#     codec = 'utf-8'
#     laparams = LAParams()
#     device = TextConverter(rsrcmgr, retstr, codec=codec, laparams=laparams)
#     fp = file(path, 'rb')
#     interpreter = PDFPageInterpreter(rsrcmgr, device)
#     password = ""
#     maxpages = 0
#     caching = True
#     pagenos = set()

#     for page in PDFPage.get_pages(fp, pagenos, maxpages=maxpages, password=password, caching=caching, check_extractable=True):
#         interpreter.process_page(page)
#     fp.close()
#     device.close()
#     str = retstr.getvalue()
#     retstr.close()
#     return str


def convert_pdf_to_txt(path):
    raise Exception("This part of the code is not yet ported to Python3")


def document_to_text(path):
    ext = os.path.splitext(path)[1]
    filename = os.path.basename(path)

    if ext == ".doc":
        cmd = os.path.dirname(os.path.realpath(
            __file__)) + os.sep + "antiword" + os.sep + "antiword.exe -m CP852 " + path
        pipe = Popen(cmd, stdout=PIPE, shell=True)
        text = pipe.communicate()[0]
        return filename, str(re.sub("\r|\n", "", text).strip())

    elif ext == ".docx":
        document = opendocx(path)
        paratextlist = getdocumenttext(document)
        newparatextlist = []
        for paratext in paratextlist:
            newparatextlist.append(paratext)
        return filename, "".join(newparatextlist)

    elif ext == ".pdf":
        text = convert_pdf_to_txt(path)
        return filename, str(re.sub("\r|\n", "", text).strip())

    else:
        text = open(path, "r").read().strip()
        return filename, text
